//
//  CFRFractalView.m
//  Mandelbrot
//
//  Created by Renaud Pradenc on 02/02/09.
//  Copyright 2009 Céroce. All rights reserved.
//

#import "CFRFractalView.h"

static void* ContexteMAJRendu = (void*) @"ContexteMAJRendu";


@implementation CFRFractalView

#pragma mark Gestion de la classe

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self)
	{
        trackingArea = nil;
		pointeurDansLaVue = NO;
    }
    return self;
}

/* Informe la vue qu'elle vient d'être insérée dans la fenêtre */
- (void)viewDidMoveToWindow
{
	// Créer une Tracking Area pour savoir quand changer le pointeur de la souris.
	trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds] 
				options:(NSTrackingMouseEnteredAndExited | NSTrackingInVisibleRect | NSTrackingActiveInKeyWindow) 
				owner:self 
				userInfo:nil];
	[self addTrackingArea:trackingArea];

}

- (void) awakeFromNib
{
	[render addObserver:self forKeyPath:@"besoinMAJ" options:0 context:ContexteMAJRendu];	
}

	- (void)viewWillMoveToWindow:(NSWindow *)newWindow
	{
		if(newWindow == nil)
			[render removeObserver:self forKeyPath:@"besoinMAJ"];
	}

- (void) dealloc
{
	[trackingArea release];
	[super dealloc];
}

#pragma mark KVO

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if(context == ContexteMAJRendu)
	{
		[self setNeedsDisplay:YES];	
	}
		
}


#pragma mark Dessin

- (void)drawRect:(NSRect)rect
{
	// L'outlet n'est pas fixée
	if(render == nil)	
	{
		[[NSColor blueColor] set];
		NSRectFill(rect);		
		return;
	}
	
	// Afficher la bitmap
	[render setLargeurBitmap:[self bounds].size.width];
	[render setHauteurBitmap:[self bounds].size.height];
	NSBitmapImageRep* bitmapRep = [render bitmapImageRep];
	[bitmapRep drawAtPoint:NSMakePoint(0,0)];	

}

#pragma mark Événements souris

// Informe la vue que la molette de la souris a tourné: zoomer la vue
- (void)scrollWheel:(NSEvent *)theEvent
{
	// deltaY est > 0 quand la molette est tournée en avant.
	//	Il vaut +/- 0.1 pour un petit mouvement et +/- 10 pour un grand.
	double deltaY = [theEvent deltaY];
	
	[render zoomerDuFacteur: 1.0 + deltaY/20.0];
}

/* Informe que la souris a été glissée avec le bouton appuyé.
	- Dans le cas nominal, il faut se déplacer dans le plan
 	- Si la touche Contrôle est appuyée, il faut zoomer/dézoomer selon le mouvement vertical de la souris
*/
- (void)mouseDragged:(NSEvent *)theEvent
{
	// La touche Contrôle est-elle appuyée ?
	if([theEvent modifierFlags] & NSControlKeyMask)
	{	
		// Zoomer/dézoomer
		double deltaRelatif = 2.0 * ([theEvent deltaY] / [self bounds].size.height);
		
		[render zoomerDuFacteur: 1.0 + deltaRelatif];
	}
	else
	{
		// Déplacer le centre de la vue
		[render decalerCentreX:-[theEvent deltaX] y:[theEvent deltaY]];		
	}
}

/* Informe la vue qu'elle a été cliquée avec le bouton principal. */
- (void)mouseDown:(NSEvent *)theEvent
{
	// La touche Contrôle est-elle appuyée ?
	if([theEvent modifierFlags] & NSControlKeyMask)
	{
		// Le curseur est certainement déjà resizeUpDown, mais ça permet d'équilibrer l'appel à 
		// [NSCursor pop] dans mouseUp:.
		[[NSCursor resizeUpDownCursor] push];	
	}
	else
	{
		// Mettre un pointeur de souris en forme de main fermée
		[[NSCursor closedHandCursor] push];	
	}
}

/* Informe la vue que le bouton principal de la souris a été relâché. */
- (void)mouseUp:(NSEvent *)theEvent
{
	[NSCursor pop];	
}


/* Informe la vue que le pointeur de la souris est entré dans une tracking area. */
- (void)mouseEntered:(NSEvent *)theEvent
{
	[[NSCursor openHandCursor] push];
	pointeurDansLaVue = YES;
}

/* Informe la vue que le pointeur de la souris est sorti d'une tracking area. */
- (void)mouseExited:(NSEvent *)theEvent
{
	pointeurDansLaVue = NO;
	[NSCursor pop];
}


#pragma mark Événements clavier

/* Informe la chaîne des répondeurs que la vue accepte les événements clavier. */
- (BOOL) acceptsFirstResponder
{
	return YES;
}

/* Informe la vue qu'une touche spéciale est appuyée ou relâchée. */
- (void)flagsChanged:(NSEvent *)theEvent
{
	if(pointeurDansLaVue)
	{
		if([theEvent modifierFlags] & NSControlKeyMask)
			[[NSCursor resizeUpDownCursor] push];	
		else
			[NSCursor pop];
	}
}

@end
