//
//  CFRMandelbrotView.m
//  Mandelbrot
//
//  Created by Renaud Pradenc on 02/02/09.
//  Copyright 2009 Céroce. All rights reserved.
//

#import "CFRMandelbrotView.h"

#define MAX_ITERATIONS	18


@implementation CFRMandelbrotView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self)
	{
        
    }
    return self;
}

- (void)drawRect:(NSRect)rect
{
	// Créer la bitmap
	NSBitmapImageRep* bitmapRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:NULL
																		  pixelsWide:400
																		  pixelsHigh:300
																	   bitsPerSample:8
																	 samplesPerPixel:1
																			hasAlpha:NO
																			isPlanar:NO
																	  colorSpaceName:NSDeviceWhiteColorSpace
																		 bytesPerRow:0
																		bitsPerPixel:8];
	
	// Calculer l'ensemble de Mandelbrot:
	// Parcourir tous les points de la bitmap
	double x, y;
	for(x = 0; x < 400; x++)
	{
		for(y = 0; y < 300; y++)
		{
			// Convertir les coordonnées
			Complexe_t c;
			c.reel =  x/100.0 - 2;
			c.imag   = -y/100.0 + 1.5;
			
			// Initialiser z[0]
			Complexe_t z = {0.0, 0.0};

			NSUInteger n;
			for(n=0; n < MAX_ITERATIONS; n++)
			{
				// z[n+1] = z[n+1]^2 + c
				Complexe_t zCarre = Carre(z);
				z.reel = zCarre.reel + c.reel;
				z.imag = zCarre.imag + c.imag;
				
				// La suite diverge si |z| > 2
				double moduleAuCarre = z.reel*z.reel + z.imag*z.imag;
				if(moduleAuCarre > 4.0)
					break;
			}
			
			// Donner le niveau de gris au pixel
			NSUInteger nuance = n * 255 / MAX_ITERATIONS;
			[bitmapRep setPixel:&nuance atX:x y:y];
		}
	}
	
	// Afficher la bitmap
	[bitmapRep drawAtPoint:NSMakePoint(0,0)];
	[bitmapRep release];
}

/* Calcule le carré d'un nombre complexe.
 	(a+ib)^2 = (a+ib)(a+ib) = a^2 + i2ab - b^2
*/
Complexe_t Carre(Complexe_t z)
{
	Complexe_t carre;
	
	carre.reel = z.reel*z.reel - z.imag*z.imag;
	carre.imag = 2.0 * z.reel * z.imag;
	
	return carre;
}


@end
